@@ -16,9 +16,9 @@ Kcom <- local({
                    trend=~1, interaction=Poisson(), rbord=reach(interaction),
                    compute.var=TRUE,
                    truecoef=NULL, hi.res=NULL) {
-  if(inherits(object, "ppm")) {
+  if(is.ppm(object)) {
     fit <- object
-  } else if(is.ppp(object) || inherits(object, "quad")) {
+  } else if(is.ppp(object) || is.quad(object)) {
     if(is.ppp(object)) object <- quadscheme(object, ...)
     if(!is.null(model)) {
       fit <- update(model, Q=object, forcefit=TRUE)
