@@ -3,13 +3,13 @@
 #
 #  Method for 'density' for point patterns
 #
-#  $Revision: 1.85 $    $Date: 2016/11/13 01:54:32 $
+#  $Revision: 1.87 $    $Date: 2017/12/30 05:00:22 $
 #
 
-ksmooth.ppp <- function(x, sigma, ..., edge=TRUE) {
-  .Deprecated("density.ppp", package="spatstat")
-  density.ppp(x, sigma, ..., edge=edge)
-}
+# ksmooth.ppp <- function(x, sigma, ..., edge=TRUE) {
+#  .Deprecated("density.ppp", package="spatstat")
+#  density.ppp(x, sigma, ..., edge=edge)
+# }
 
 density.ppp <- local({
   
@@ -19,7 +19,7 @@ density.ppp <- function(x, sigma=NULL, ...,
                         adjust=1, diggle=FALSE, se=FALSE, 
                         kernel="gaussian",
                         scalekernel=is.character(kernel),
-                        positive=FALSE) {
+                        positive=FALSE, verbose=TRUE) {
   verifyclass(x, "ppp")
 
   output <- pickoption("output location type", at,
@@ -31,7 +31,7 @@ density.ppp <- function(x, sigma=NULL, ...,
     ## kernel is only partly implemented!
     if(se)
       stop("Standard errors are not implemented for non-Gaussian kernel")
-    if(is.function(sigma) || (is.null(sigma) && is.null(varcov)))
+    if(verbose && (is.function(sigma) || (is.null(sigma) && is.null(varcov))))
       warning("Bandwidth selection will be based on Gaussian kernel")
   }
   
@@ -65,7 +65,7 @@ density.ppp <- function(x, sigma=NULL, ...,
                                   weights=weights, edge=edge,
                                   leaveoneout=leaveoneout,
                                   diggle=diggle, ...)
-    if(!is.null(uhoh <- attr(result, "warnings"))) {
+    if(verbose && !is.null(uhoh <- attr(result, "warnings"))) {
       switch(uhoh,
              underflow=warning("underflow due to very small bandwidth"),
              warning(uhoh))
