@@ -1,7 +1,7 @@
 #
 # pppmatch.R
 #
-# $Revision: 1.22 $  $Date: 2014/11/11 01:06:44 $
+# $Revision: 1.24 $  $Date: 2018/04/17 02:14:31 $
 #
 # Code by Dominic Schuhmacher
 #
@@ -42,14 +42,13 @@ pppmatching <- function(X, Y, am, type = NULL, cutoff = NULL,
       stop("Adjacency matrix does not have the right dimensions")
    am <- matrix(as.numeric(am), n1, n2)
    #am <- apply(am, c(1,2), as.numeric)
-   res <- list("pp1" = X, "pp2" = Y, "matrix" = am, "type" = type, "cutoff" = cutoff, 
-      "q" = q, "distance" = mdist)
+   res <- list("pp1" = X, "pp2" = Y, "matrix" = am,
+               "type" = type, "cutoff" = cutoff, 
+               "q" = q, "distance" = mdist)
    class(res) <- "pppmatching"
    res
 }
 
-# currently, for fractional matchings all the flows are plotted the same way
-# irrespective of their weights
 plot.pppmatching <- function(x, addmatch = NULL, main = NULL, ...) {
    if (is.null(main))
       main <- short.deparse(substitute(x))
@@ -64,7 +63,8 @@ plot.pppmatching <- function(x, addmatch = NULL, main = NULL, ...) {
    }
    if (length(here) > 0) {
      seg <- as.psp(from=pp1[here[,1]], to=pp2[here[,2]])
-     plot(seg, add=TRUE, ...)
+     marks(seg) <- x$matrix[here]
+     plot(seg, add=TRUE, ..., style="width")
    }
    points(x$pp1, pch=20, col=2, ...)
    points(x$pp2, pch=20, col=4, ...)
@@ -75,24 +75,25 @@ print.pppmatching <- function(x, ...) {
    n1 <- x$pp1$n
    n2 <- x$pp2$n
    if (is.null(x$type) || is.null(x$q) || is.null(x$cutoff))
-     cat("Generic matching of two planar point patterns \n")
+     splat("Generic matching of two planar point patterns")
    else
-     cat(x$type, "-", x$q, " matching of two planar point patterns (cutoff = ",
-       x$cutoff, ") \n", sep = "")
-   cat("pp1:", n1, ngettext(n1, "point", "points"), "\n")
-   cat("pp2:", n2, ngettext(n2, "point", "points"), "\n")
-   print.owin(x$pp1$window)
+     splat(x$type, "-",
+           x$q, " matching of two planar point patterns (cutoff = ",
+           x$cutoff, ")", sep = "")
+   splat("pp1:", n1, ngettext(n1, "point", "points"))
+   splat("pp2:", n2, ngettext(n2, "point", "points"))
+   print(Window(x$pp1))
    npair <- sum(x$matrix > 0)
    if (npair == 0)
-     cat("matching is empty \n") 
+     splat("matching is empty")
    else {
      if (any(x$matrix != trunc(x$matrix)))
-       cat("fractional matching,", npair, ngettext(npair, "flow", "flows"), "\n")
+       splat("fractional matching,", npair, ngettext(npair, "flow", "flows"))
      else
-       cat("point matching,", npair, ngettext(npair, "line", "lines"), "\n")
+       splat("point matching,", npair, ngettext(npair, "line", "lines"))
    }
    if (!is.null(x$distance))
-     cat("distance:", x$distance, "\n") 
+     splat("distance:", x$distance)
    return(invisible(NULL))
 }
 
@@ -102,22 +103,23 @@ summary.pppmatching <- function(object, ...) {
    n1 <- X$n
    n2 <- Y$n
    if (is.null(object$type) || is.null(object$q) || is.null(object$cutoff))
-     cat("Generic matching of two planar point patterns \n")
+     splat("Generic matching of two planar point patterns")
    else
-     cat(object$type, "-", object$q, " matching of two planar point patterns (cutoff = ",
-       object$cutoff, ") \n", sep = "")
-   cat("pp1:", n1, ngettext(n1, "point", "points"), "\n")
-   cat("pp2:", n2, ngettext(n2, "point", "points"), "\n")
-   print.owin(X$window)
+     splat(object$type, "-", object$q,
+           " matching of two planar point patterns (cutoff = ",
+           object$cutoff, ")", sep = "")
+   splat("pp1:", n1, ngettext(n1, "point", "points"))
+   splat("pp2:", n2, ngettext(n2, "point", "points"))
+   print(Window(X))
    npair <- sum(object$matrix > 0)
    if (npair == 0)
-     cat("matching is empty \n") 
+     splat("matching is empty") 
    else {
      if (any(object$matrix != trunc(object$matrix))) {
-       cat("fractional matching,", npair, ngettext(npair, "flow", "flows"), "\n")
+       splat("fractional matching,", npair, ngettext(npair, "flow", "flows"))
      }
      else {
-       cat("point matching,", npair, ngettext(npair, "line", "lines"), "\n")
+       splat("point matching,", npair, ngettext(npair, "line", "lines"))
        rowsum <- rowSums(object$matrix)
        colsum <- colSums(object$matrix)
        lt <- ifelse(min(rowsum) >= 1, TRUE, FALSE)
@@ -125,19 +127,19 @@ summary.pppmatching <- function(object, ...) {
        rt <- ifelse(min(colsum) >= 1, TRUE, FALSE)
        lu <- ifelse(max(colsum) <= 1, TRUE, FALSE)
        if (lt && ru && rt && lu)
-         cat("matching is 1-1 \n")
+         splat("matching is 1-1")
        else if (any(lt, ru, rt, lu)) {
-         cat("matching is",
+         splat("matching is",
                    ifelse(lt, " left-total", ""),
                    ifelse(lu, " left-unique", ""),
                    ifelse(rt, " right-total", ""),
                    ifelse(ru, " right-unique", ""),
-                   "\n", sep="")
+                   sep="")
          }
      }
    }
    if (!is.null(object$distance))
-     cat("distance:", object$distance, "\n") 
+     splat("distance:", object$distance)
    return(invisible(NULL))
 }
 
