@@ -3,7 +3,7 @@
 ##
 ## Exact 'funxy' counterpart of Smooth.ppp
 ##
-##  $Revision: 1.2 $ $Date: 2016/02/11 10:17:12 $
+##  $Revision: 1.6 $ $Date: 2018/04/13 06:24:43 $
 
 
 Smoothfun <- function(X, ...) {
@@ -15,8 +15,25 @@ Smoothfun.ppp <- function(X, sigma=NULL, ...,
   verifyclass(X, "ppp")
   if(!is.marked(X, dfok=TRUE))
     stop("X should be a marked point pattern")
-  stuff <- list(X=X, weights=weights, edge=edge, diggle=diggle)
+  ## handle weights now
+  weightsgiven <- !missing(weights) && !is.null(weights) 
+  if(weightsgiven) {
+    # convert to numeric
+    if(is.im(weights)) {
+      weights <- safelookup(weights, X) # includes warning if NA
+    } else if(is.expression(weights)) 
+      weights <- eval(weights, envir=as.data.frame(X), enclos=parent.frame())
+    if(length(weights) == 0)
+      weightsgiven <- FALSE
+  }
+  if(weightsgiven) {
+    check.nvector(weights, npoints(X))
+  } else weights <- NULL
+  ## 
   X <- coerce.marks.numeric(X)
+  ## 
+  stuff <- list(X=X, weights=weights, edge=edge, diggle=diggle)
+  ## 
   ## determine smoothing parameters
   ker <- resolve.2D.kernel(sigma=sigma, ...,
                            x=X, bwfun=bw.smoothppp, allow.zero=TRUE)
@@ -49,10 +66,17 @@ print.Smoothfun <- function(x, ...) {
 ## Method for as.im
 ## (enables plot.funxy, persp.funxy, contour.funxy to work for this class)
 
-as.im.Smoothfun <- function(X, W=NULL, ...) {
+as.im.Smoothfun <- function(X, W=Window(X), ..., approx=TRUE) {
   stuff <- get("stuff", envir=environment(X))
-  if(!is.null(W)) stuff$X <- stuff$X[W]
-  do.call(Smooth, resolve.defaults(list(...), stuff))
+  if(!approx) {
+    #' evaluate exactly at grid points 
+    result <- as.im.function(X, W=W, ...)
+  } else {
+    #' faster, approximate evaluation using FFT
+    if(!is.null(W)) stuff$X <- stuff$X[W]
+    result <- do.call(Smooth, resolve.defaults(list(...), stuff))
+  }
+  return(result)
 }
 
 
