@@ -3,7 +3,7 @@
 #
 # support for tessellations
 #
-#   $Revision: 1.78 $ $Date: 2018/03/07 04:06:58 $
+#   $Revision: 1.85 $ $Date: 2018/05/29 09:15:07 $
 #
 tess <- function(..., xgrid=NULL, ygrid=NULL, tiles=NULL, image=NULL,
                  window=NULL, marks=NULL, keepempty=FALSE,
@@ -62,10 +62,10 @@ tess <- function(..., xgrid=NULL, ygrid=NULL, tiles=NULL, image=NULL,
     lev <- if(!is.null(nam) && all(nzchar(nam))) nam else 1:ntiles
     if(is.null(window)) 
       window <- do.call(union.owin, unname(tiles))
-    if(is.mask(window) || any(unlist(lapply(tiles, is.mask)))) {
+    if(is.mask(window) || any(sapply(tiles, is.mask))) {
       # convert to pixel image tessellation
-      window <- as.mask(window)
-      ima <- as.im(window)
+      Grid <- do.call(commonGrid, append(list(window), unname(tiles)))
+      ima <- as.im(window, W=Grid)
       ima$v[] <- NA
       for(i in 1:ntiles)
         ima[tiles[[i]]] <- i
@@ -192,25 +192,58 @@ plot.tess <- local({
                         labelargs=list(),
                         do.col=FALSE, 
                         values=marks(x),
+                        multiplot=TRUE,
                         col=NULL,
                         ribargs=list()) {
     if(missing(main) || is.null(main))
       main <- short.deparse(substitute(x))
-    if(do.col) {
+    ntiles <- x$n
+    if(!do.col) {
+      #' Plot tiles, with adornment
+      y <- NULL
+      result <- NULL
+      bbox <- NULL
+      need.legend <- FALSE
+    } else {
+      #' Fill tiles with colours determined by 'values'
+      if(markformat(values) == "hyperframe") 
+        values <- as.data.frame(values) #' automatic warning
       #' Determine values associated with each tile
-      if(is.null(values)) {
-        #' default is tile ID
-        values <- factor(seq_len(x$n))
-      } else {
-        if(is.data.frame(values)) {
-          if(ncol(values) > 1)
-            warning("Using only the first column of values")
-          values <- values[,1]
-        }
-        if(length(values) != x$n)
-          stop(paste("Number of values =", length(values),
-                     "!=", x$n, "= number of tiles"))
-      }
+      switch(markformat(values),
+             none = {
+               #' no values assigned.
+               #' default is tile name
+               values <- factor(tilenames(x))
+             },
+             vector = {
+               #' vector of values.
+               #' validate length of vector
+               check.nvector(values, ntiles, things="tiles")
+             },
+             dataframe = {
+               #' data frame or matrix of values.
+               values <- as.data.frame(values)
+               if(nrow(values) != ntiles)
+                 stop(paste("Number of rows of values =", nrow(values),
+                            "!=", ntiles, "= number of tiles"),
+                      call.=FALSE)
+               if(multiplot && ncol(values) > 1 && !add) {
+                 #' Multiple Panel Plot
+                 result <- multi.plot.tess(x, ...,
+                                           main=main, show.all=show.all,
+                                           border=border, do.plot=do.plot,
+                                           do.labels=do.labels, labels=labels,
+                                           labelargs=labelargs, do.col=do.col, 
+                                           col=col, ribargs=ribargs)
+                 return(invisible(result))
+               }
+               if(ncol(values) > 1)
+                 warning("Using only the first column of values")
+               values <- values[,1]
+             },
+             stop("Format of values is not understood")
+             )
+      #' Single Panel Plot
       #' Determine colour map and plan layout (including colour ribbon)
       #' using rules for pixel images
       y <- as.im(as.function(x, values=values))
@@ -220,22 +253,21 @@ plot.tess <- local({
                                do.plot=FALSE,
                                show.all=show.all, add=add, main=main,
                                col=col, ribargs=ribargs),
-                          list(...)))
+                          list(...),
+                          list(valuesAreColours=FALSE)
+                          ))
       #' exit if not actually plotting
-      if(!do.plot) return(result)
+      if(!do.plot) return(invisible(result))
       #' extract info
       colmap <- result
       bbox <- attr(result, "bbox")
       bbox.legend <- attr(result, "bbox.legend")
-      need.legend <- TRUE
-    } else {
-      result <- NULL
-      bbox <- NULL
-      need.legend <- FALSE
+      need.legend <- !is.null(bbox.legend)
     }
+    #'      Start Plot 
     #' initialise plot region if it is determined
     if(do.plot && !is.null(bbox) && !add) {
-      plot(bbox, main=main, type="n")
+      plot(bbox, main=" ", type="n")
       add <- TRUE
     }
     switch(x$type,
@@ -305,9 +337,10 @@ plot.tess <- local({
              }
            },
            image={
+             if(is.null(y)) y <- x$image
              result <-
                do.call(plot,
-                       resolve.defaults(list(x$image, add=add, main=main,
+                       resolve.defaults(list(y, add=add, main=main,
                                              show.all=show.all,
                                              do.plot=do.plot,
                                              col=col, ribargs=ribargs),
@@ -344,6 +377,17 @@ plot.tess <- local({
     return(invisible(result))
   }
 
+  multi.plot.tess <- function(x, ..., zlim=NULL, col=NULL, equal.ribbon=FALSE) {
+    if(equal.ribbon && is.null(zlim) && !inherits(col, "colourmap"))
+      zlim <- range(marks(x))
+    if(!is.null(zlim)) {
+      result <- plot(unstack(x), ..., zlim=zlim, col=col)
+    } else {
+      result <- plot(unstack(x), ..., col=col)
+    }
+    return(invisible(result))
+  }
+  
   plot.tess
 })
 
@@ -391,28 +435,24 @@ tiles <- function(x) {
            yg <- x$ygrid
            nx <- length(xg) - 1
            ny <- length(yg) - 1
-           for(j in rev(seq_len(ny)))
+           for(j in rev(seq_len(ny))) {
              for(i in seq_len(nx)) {
                winij <- owin(xg[c(i,i+1)], yg[c(j,j+1)])
-               dout <- list(winij)
-               names(dout) <- paste("Tile row ", ny-j+1, ", col ", i,
-                                    sep="")
-               out <- append(out, dout)
+               out <- append(out, list(winij))
              }
+           }
          },
          tiled={
            out <- x$tiles
-           if(is.null(names(out)))
-             names(out) <- paste("Tile", seq_along(out))
-         },
+           },
          image={
            out <- list()
            ima <- x$image
            lev <- levels(ima)
            for(i in seq_along(lev))
              out[[i]] <- solutionset(ima == lev[i])
-           names(out) <- paste(lev)
-         })
+           })
+  names(out) <- tilenames(x)
   out <- as.solist(out)
   return(out)
 }
@@ -969,3 +1009,29 @@ as.data.frame.tess <- function(x, ...) {
          })
   return(z)
 }
+
+connected.tess <- function(X, ...) {
+  Xim <- as.im(X, ...)
+  X <- as.tess(Xim)
+  tilesX <- tiles(X)
+  namesX <- names(tilesX)
+  shards <- lapply(tilesX, connected) # list of factor images
+  shardnames <- lapply(shards, levels)
+  nshards <- lengths(shardnames)
+  broken <- (nshards > 1)
+  #' unbroken tiles keep their original tile names
+  shardnames[!broken] <- namesX[!broken]
+  #' shards of broken tiles are named "tilename[i] shard j"
+  shardnames[broken] <- mapply(paste,
+                               namesX[broken], "shard", shardnames[broken],
+                               SIMPLIFY=FALSE)
+  #' rename them
+  shards <- mapply("levels<-", shards, shardnames, SIMPLIFY=FALSE)
+  #' separate them
+  shards <- lapply(lapply(shards, as.tess), tiles)
+  shards <- unlist(shards, recursive=FALSE, use.names=FALSE)
+  names(shards) <- unlist(shardnames)
+  #' form tessellation
+  result <- tess(tiles=shards, window=as.owin(Xim))
+  result
+}
